@@ -18,18 +18,24 @@ Examples
 
   soops-run -r 1 -n 3 -c='--switch + --seed' -o output "python='python3', output_dir='output/study/%s', --num=[100,1000,10000], --repeat=[10,20], --switch=['@undefined', '@defined', '@undefined', '@defined'], --seed=['@undefined', '@undefined', 12345, 12345], --host=['random', 'first'], --silent=@defined, --no-show=@defined" examples/monty_hall.py
 
-   soops-info -e output/study/1_0_2_0_1_3_0_3_0/ examples/monty_hall.py
+  soops-info examples/monty_hall.py -e output/study/00*
 
-  soops-scoop examples/monty_hall.py output/study/ -s rdir -o output/study -r output/study/results.h5
+  soops-scoop examples/monty_hall.py output/study/ -s rdir -o output/study
 
-  soops-scoop examples/monty_hall.py output/study/ -s rdir -o output/study -r output/study/results.h5 --plugin-args=plot_win_rates={colormap_name='plasma'}
+  soops-scoop examples/monty_hall.py output/study/ -s rdir -o output/study -r --plugin-args=plot_win_rates={colormap_name='plasma'}
 
-  soops-scoop examples/monty_hall.py output/study/ -o output/study -r output/study/results.h5 --no-plugins --shell
+  soops-scoop examples/monty_hall.py output/study/ -o output/study -r --no-plugins --shell
+
+- Use --generate-pars instead of listing values of --seed and --switch::
+
+  soops-run -r 1 -n 3 -c='--switch + --seed' -o output/study2 "python='python3', output_dir='output/study2/%s', --num=[100,1000,10000], --repeat=[10,20], --switch=@generate, --seed=@generate, --host=['random', 'first'], --silent=@defined, --no-show=@defined" --generate-pars="function=generate_seed_switch, seeds=['@undefined', 12345], switches=['@undefined', '@defined']" examples/monty_hall.py
+
+  soops-scoop examples/monty_hall.py output/study2/0* -s rdir -o output/study2
 """
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
 import os
-import time
 from functools import partial
+from itertools import product
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -63,6 +69,23 @@ def get_run_info():
 
     return run_cmd, opt_args, output_dir_key, is_finished_basename
 
+def generate_seed_switch(args, gkeys, dconf, options):
+    """
+    Parameters
+    ----------
+    args : Struct
+        The arguments passed from the command line.
+    gkeys : list
+        The list of option keys to generate.
+    dconf : dict
+        The parsed parameters of the parametric study.
+    options : Namespace
+        The soops-run command line options.
+    """
+    seeds, switches = zip(*product(args.seeds, args.switches))
+    gconf = {'--seed' : list(seeds), '--switch' : list(switches)}
+    return gconf
+
 def get_scoop_info():
     info = [
         ('options.txt', partial(
@@ -213,7 +236,7 @@ def main():
         if options.seed is not None:
             np.random.seed(options.seed)
 
-        tt = time.perf_counter()
+        timer = so.Timer().start()
         history = []
         choices = {0, 1, 2}
         for ii in range(options.num):
@@ -241,7 +264,7 @@ def main():
             win = doors[choice2]
             history.append(win)
 
-        output('elapsed:', time.perf_counter() - tt)
+        output('elapsed:', timer.stop())
         output('win rate:', sum(history) / options.num)
 
         histories.append(history)
