@@ -5,6 +5,7 @@ Scoop output files.
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
 import sys
 import os.path as op
+import glob
 from datetime import datetime
 import warnings
 
@@ -43,7 +44,16 @@ def load_split_options(filename, split_keys=None, rdata=None):
         options = split_options(options, split_keys=split_keys)
     return options
 
-def apply_scoops(info, directories):
+def filter_dict(data, key_prefix, strip_prefix=True):
+    """
+    Filter a subset of `data` with keys starting with `key_prefix`.
+    """
+    out = Struct((key.replace(key_prefix, '', 1) if strip_prefix else key, val)
+                 for key, val in data.items()
+                 if key.startswith(key_prefix))
+    return out
+
+def apply_scoops(info, directories, debug_mode=False):
     if not len(info):
         return pd.DataFrame({}), pd.DataFrame({}), None
 
@@ -60,7 +70,7 @@ def apply_scoops(info, directories):
             rdir = op.dirname(filename)
             output('results directory {}: {}'.format(ir, rdir))
 
-            rdata = {'rdir' : rdir.replace(home, '~')}
+            rdata = {'rdir' : rdir.replace(home, '~'), 'rfiles' : []}
             rmetadata = {}
             output('results files:')
             for item in info:
@@ -77,8 +87,22 @@ def apply_scoops(info, directories):
 
                 output(filename)
                 path = op.join(rdir, filename)
+                if not op.exists(path):
+                    paths = list(locate_files(path))
+                    output('expanded:', [path.replace(rdir, '<rdir>')
+                                         for path in paths])
+                    if len(paths) == 0:
+                        paths = None
+
+                else:
+                    paths = None
+
                 try:
-                    out = fun(path, rdata=rdata)
+                    if paths is None:
+                        out = fun(path, rdata=rdata)
+
+                    else:
+                        out = fun(paths, rdata=rdata)
 
                 except KeyboardInterrupt:
                     raise
@@ -86,20 +110,34 @@ def apply_scoops(info, directories):
                 except Exception as exc:
                     output('- failed with:')
                     output(exc)
+                    if debug_mode: raise
                     continue
 
                 else:
-                    try:
-                        mtime = datetime.fromtimestamp(op.getmtime(path))
+                    if out is None:
+                        output('- nothing returned!')
+                        out = {}
 
-                    except FileNotFoundError:
-                        mtime = np.nan
+                    if paths is None:
+                        paths = [path]
+
+                    rdata['rfiles'].append(filename)
+                    mtimes = []
+                    for path in paths:
+                        try:
+                            mtime = datetime.fromtimestamp(op.getmtime(path))
+
+                        except FileNotFoundError:
+                            mtime = np.nan
+
+                        mtimes.append(mtime)
 
                     rmetadata.update({
                         'data_row' : len(data),
                         'data_columns' : tuple(out.keys()),
                         'filename' : path,
-                        'mtime' : mtime,
+                        'filenames' : paths,
+                        'mtimes' : mtimes,
                     })
                     rdata.update(out)
                     metadata.append(pd.Series(rmetadata))
@@ -129,7 +167,21 @@ def get_uniques(df, columns):
 
     return uniques
 
-def run_plugins(info, df, output_dir, par_keys, plugin_args=None):
+def init_plugin_data(df, par_keys, output_dir, store_filename):
+    par_uniques = get_uniques(df, par_keys)
+    multi_par_keys = [key for key, vals in par_uniques.items()
+                      if len(vals) > 1]
+    multi_par_uniques = {key : par_uniques[key] for key in multi_par_keys}
+    data = Struct(par_keys=par_keys,
+                  multi_par_keys=multi_par_keys,
+                  par_uniques=par_uniques,
+                  multi_par_uniques=multi_par_uniques,
+                  output_dir=output_dir,
+                  store_filename=store_filename)
+    return data
+
+def run_plugins(info, df, output_dir, par_keys, store_filename,
+                plugin_args=None):
     if not len(info):
         return
 
@@ -152,15 +204,7 @@ def run_plugins(info, df, output_dir, par_keys, plugin_args=None):
         return _fun
 
     output('run plugins:')
-    par_uniques = get_uniques(df, par_keys)
-    multi_par_keys = [key for key, vals in par_uniques.items()
-                      if len(vals) > 1]
-    multi_par_uniques = {key : par_uniques[key] for key in multi_par_keys}
-    data = Struct(par_keys=par_keys,
-                  multi_par_keys=multi_par_keys,
-                  par_uniques=par_uniques,
-                  multi_par_uniques=multi_par_uniques,
-                  output_dir=output_dir)
+    data = init_plugin_data(df, par_keys, output_dir, store_filename)
     for fun in info:
         output('running {}()...'.format(fun.__name__))
         wfun = wrap_fun(fun)
@@ -172,7 +216,7 @@ def run_plugins(info, df, output_dir, par_keys, plugin_args=None):
 
 helps = {
     'sort' : 'column keys for sorting of DataFrame rows',
-    'results' : 'reuse previously scooped results file',
+    'filter' : 'use only DataFrame rows with given files successfully scooped',
     'no_plugins' : 'do not call post-processing plugins',
     'use_plugins' : 'use only the named plugins (no effect with --no-plugins)',
     'omit_plugins' : 'omit the named plugins (no effect with --no-plugins)',
@@ -181,21 +225,30 @@ helps = {
     'plugin_args' :
     """optional arguments passed to plugins given as plugin_name={key1=val1,
        key2=val2, ...}, ...""",
+    'results' : 'results file name [default: <output_dir>/results.h5]',
+    'no_csv' : 'do not save results as CSV (use only HDF5)',
+    'reuse' : 'reuse previously scooped results file',
+    'write' : 'write results files even when results were loaded using '
+    '--reuse option',
     'shell' : 'run ipython shell after all computations',
+    'debug' : 'automatically start debugger when an exception is raised',
     'output_dir' : 'output directory [default: %(default)s]',
     'scoop_mod' : 'the importable script/module with get_scoop_info()',
-    'directories' : 'results directories',
+    'directories' :
+    """results directories. On "Argument list too long" system error,
+       enclose the directories matching pattern in "", it will be expanded
+       using glob.glob().""",
 }
 
 def parse_args(args=None):
     parser = ArgumentParser(description=__doc__,
                             formatter_class=RawDescriptionHelpFormatter)
-    parser.add_argument('-s', '--sort', metavar='column[,columns,...]',
+    parser.add_argument('-s', '--sort', metavar='column[,column,...]',
                         action='store', dest='sort',
                         default=None, help=helps['sort'])
-    parser.add_argument('-r', '--results', metavar='filename',
-                        action='store', dest='results',
-                        default=None, help=helps['results'])
+    parser.add_argument('--filter', metavar='filename[,filename,...]',
+                        action='store', dest='filter',
+                        default=None, help=helps['filter'])
     parser.add_argument('--no-plugins',
                         action='store_false', dest='call_plugins',
                         default=True, help=helps['no_plugins'])
@@ -212,9 +265,24 @@ def parse_args(args=None):
     parser.add_argument('--plugin-args', metavar='dict-like',
                         action='store', dest='plugin_args',
                         default=None, help=helps['plugin_args'])
+    parser.add_argument('--results', metavar='filename',
+                        action='store', dest='results',
+                        default=None, help=helps['results'])
+    parser.add_argument('--no-csv',
+                        action='store_false', dest='save_csv',
+                        default=True, help=helps['no_csv'])
+    parser.add_argument('-r', '--reuse',
+                        action='store_true', dest='reuse',
+                        default=False, help=helps['reuse'])
+    parser.add_argument('--write',
+                        action='store_true', dest='write',
+                        default=False, help=helps['write'])
     parser.add_argument('--shell',
                         action='store_true', dest='shell',
                         default=False, help=helps['shell'])
+    parser.add_argument('--debug',
+                        action='store_true', dest='debug',
+                        default=False, help=helps['debug'])
     parser.add_argument('-o', '--output-dir', metavar='path',
                         action='store', dest='output_dir',
                         default='.', help=helps['output_dir'])
@@ -222,8 +290,14 @@ def parse_args(args=None):
     parser.add_argument('directories', nargs='+', help=helps['directories'])
     options = parser.parse_args(args=args)
 
+    if options.debug:
+        from soops import debug_on_error; debug_on_error()
+
     options.sort = parse_as_list(options.sort)
 
+    if options.filter is not None:
+        options.filter = set(parse_as_list(options.filter, free_word=True))
+
     if options.use_plugins is not None:
         options.use_plugins = parse_as_list(options.use_plugins)
 
@@ -233,6 +307,15 @@ def parse_args(args=None):
     if options.plugin_args is not None:
         options.plugin_args = parse_as_dict(options.plugin_args)
 
+    if options.results is None:
+        options.results = op.join(options.output_dir, 'results.h5')
+
+    directories = []
+    for directory in options.directories:
+        expanded = glob.glob(directory + op.sep)
+        directories.extend(expanded)
+    options.directories = directories
+
     return options
 
 def scoop_outputs(options):
@@ -240,8 +323,9 @@ def scoop_outputs(options):
 
     scoop_mod = import_file(options.scoop_mod)
 
-    if (options.results is None
+    if (not options.reuse
         or not (op.exists(options.results) and op.isfile(options.results))):
+        new_results = True
 
         if hasattr(scoop_mod, 'get_scoop_info'):
             scoop_info = scoop_mod.get_scoop_info()
@@ -251,12 +335,30 @@ def scoop_outputs(options):
                    .format(options.scoop_mod))
             return
 
-        df, mdf, par_keys = apply_scoops(scoop_info, options.directories)
+        df, mdf, par_keys = apply_scoops(scoop_info, options.directories,
+                                         options.debug)
+
+        if options.filter is not None:
+            idf = [ii for ii, rfiles in df['rfiles'].items()
+                   if options.filter.intersection(rfiles)]
+            df = df.iloc[idf]
+            df.index = np.arange(len(df))
+
+            imdf = [ii for ii, data_row in mdf['data_row'].items()
+                    if data_row in idf]
+            mdf = mdf.iloc[imdf]
+            mdf.index = np.arange(len(mdf))
 
     else:
-        df = pd.read_hdf(options.results, 'df')
-        mdf = pd.read_hdf(options.results, 'mdf')
-        par_keys = set(pd.read_hdf(options.results, 'par_keys').to_list())
+        new_results = False
+        with pd.HDFStore(options.results, mode='r') as store:
+            df = store.get('df')
+            mdf = store.get('mdf')
+            par_keys = set(store.get('par_keys').to_list())
+            std_keys = ('/df', '/mdf', '/par_keys')
+            user_keys = set(store.keys()).difference(std_keys)
+            output('user data:')
+            output(user_keys)
 
     output('data keys:')
     output(df.keys())
@@ -270,18 +372,20 @@ def scoop_outputs(options):
     warnings.simplefilter(action='ignore',
                           category=pd.errors.PerformanceWarning)
 
-    filename = op.join(options.output_dir, 'results.csv')
-    ensure_path(filename)
-    df.to_csv(filename)
-    filename = op.join(options.output_dir, 'results-meta.csv')
-    mdf.to_csv(filename)
+    results_filename = options.results
+    ensure_path(results_filename)
+    if new_results or options.write:
+        with pd.HDFStore(results_filename, mode='w') as store:
+            store.put('df', df)
+            store.put('mdf', mdf)
+            store.put('par_keys', pd.Series(list(par_keys)))
 
-    filename = op.join(options.output_dir, 'results.h5')
-    store = pd.HDFStore(filename, mode='w')
-    store.put('df', df)
-    store.put('mdf', mdf)
-    store.put('par_keys', pd.Series(list(par_keys)))
-    store.close()
+        if options.save_csv:
+            filename = op.join(options.output_dir, 'results.csv')
+            df.to_csv(filename)
+
+        filename = op.join(options.output_dir, 'results-meta.csv')
+        mdf.to_csv(filename)
 
     if options.call_plugins:
         if options.plugin_mod is not None:
@@ -292,17 +396,29 @@ def scoop_outputs(options):
 
         if hasattr(plugin_mod, 'get_plugin_info'):
             plugin_info = plugin_mod.get_plugin_info()
-            output('available plugins:', [fun.__name__ for fun in plugin_info])
+            fun_names = [fun.__name__ for fun in plugin_info]
+            output('available plugins:', fun_names)
 
             if options.use_plugins is not None:
-                plugin_info = [fun for fun in plugin_info
-                               if fun.__name__ in options.use_plugins]
+                aux = []
+                for fun_name in options.use_plugins:
+                    try:
+                        ii = fun_names.index(fun_name)
+
+                    except ValueError:
+                        raise ValueError('unknown plugin! ({})'
+                                         .format(fun_name))
+
+                    aux.append(plugin_info[ii])
+
+                plugin_info = aux
 
             elif options.omit_plugins is not None:
                 plugin_info = [fun for fun in plugin_info
                                if fun.__name__ not in options.omit_plugins]
 
             data = run_plugins(plugin_info, df, options.output_dir, par_keys,
+                               results_filename,
                                plugin_args=options.plugin_args)
             output('plugin data keys:')
             output(data.keys())
